@@ -15,6 +15,7 @@
 #![feature(specialization)]
 #![feature(trusted_len)]
 
+pub use crate::errors::PushCapacityError;
 pub use crate::iterators::*;
 pub use crate::trait_impls::*;
 use crate::utils::*;
@@ -38,6 +39,8 @@ use alloc::vec::Vec;
 mod iterators;
 #[macro_use]
 mod macros;
+#[doc(hidden)]
+mod errors;
 mod trait_impls;
 #[doc(hidden)]
 pub mod utils;
@@ -436,22 +439,22 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Pushes `value` to the StaticVec if its current length is less than its capacity,
   /// or returns an error indicating there's no remaining capacity otherwise.
   #[inline(always)]
-  pub fn try_push(&mut self, value: T) -> Result<(), &'static str> {
+  pub fn try_push(&mut self, value: T) -> Result<(), PushCapacityError<T, N>> {
     if self.length < N {
-      unsafe {
-        self.push_unchecked(value);
-      }
-      return Ok(());
+      unsafe { self.push_unchecked(value) };
+      Ok(())
+    } else {
+      Err(PushCapacityError::new(value))
     }
-    Err("Insufficient remaining capacity!")
   }
 
   /// Pushes a value to the end of the StaticVec. Panics if the collection is
   /// full; that is, if `self.len() == self.capacity()`.
   #[inline(always)]
   pub fn push(&mut self, value: T) {
-    assert!(self.length < N, "Insufficient remaining capacity!");
-    unsafe { self.push_unchecked(value) }
+    self
+      .try_push(value)
+      .expect("Insufficient remaining capacity for push!")
   }
 
   /// Removes the value at the last position of the StaticVec and returns it in `Some` if
@@ -537,7 +540,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   }
 
   /// Returns `None` if `index` is greater than or equal to the current length of the StaticVec.
-  /// Otherwise, removes the value at that position and returns it in `Some`, and then
+  /// otherwise, removes the value at that position and returns it in `Some`, and then
   /// moves the last value in the StaticVec into the empty slot.
   #[inline(always)]
   pub fn swap_pop(&mut self, index: usize) -> Option<T> {
@@ -609,7 +612,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Returns a [`StaticVecIterConst`](crate::iterators::StaticVecIterConst) over the StaticVec's
   /// inhabited area.
   #[inline(always)]
-  pub fn iter<'a>(&'a self) -> StaticVecIterConst<'a, T, N> {
+  pub fn iter(&self) -> StaticVecIterConst<T, N> {
     StaticVecIterConst {
       start: self.as_ptr(),
       end: match intrinsics::size_of::<T>() {
@@ -623,7 +626,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Returns a [`StaticVecIterMut`](crate::iterators::StaticVecIterMut) over the StaticVec's
   /// inhabited area.
   #[inline(always)]
-  pub fn iter_mut<'a>(&'a mut self) -> StaticVecIterMut<'a, T, N> {
+  pub fn iter_mut(&mut self) -> StaticVecIterMut<T, N> {
     StaticVecIterMut {
       start: self.as_mut_ptr(),
       end: match intrinsics::size_of::<T>() {
@@ -747,6 +750,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// it will have no contents and a length of 0 afterwards.)
   #[cfg(feature = "std")]
   #[doc(cfg(feature = "std"))]
+  #[allow(clippy::wrong_self_convention)]
   #[inline(always)]
   pub fn into_vec(&mut self) -> Vec<T> {
     let mut res = Vec::with_capacity(N);
@@ -892,6 +896,83 @@ impl<T, const N: usize> StaticVec<T, N> {
     self.dedup_by(|a, b| key(a) == key(b))
   }
 
+  /// Returns a new StaticVec representing the difference of `self` and `other` (that is,
+  /// all items present in `self`, but *not* present in `other`.)
+  ///
+  /// The `N2` parameter does not need to be provided explicitly, and can be inferred from `other`
+  /// itself.
+  ///
+  /// Locally requires that `T` implements [`Clone`](core::clone::Clone) to avoid soundness issues
+  /// while accommodating for more types than [`Copy`](core::marker::Copy) would appropriately for
+  /// this function, and [`PartialEq`](core::cmp::PartialEq) to make the item comparisons possible.
+  ///
+  /// Example usage:
+  /// ```
+  /// assert_eq!(
+  ///   staticvec![4, 5, 6, 7].difference(&staticvec![1, 2, 3, 7]),
+  ///   [4, 5, 6]
+  /// );
+  /// ```
+  #[inline]
+  pub fn difference<const N2: usize>(&self, other: &StaticVec<T, N2>) -> Self
+  where T: Clone + PartialEq {
+    let min_length = self.length.min(other.length);
+    let right = other.as_slice();
+    let mut res = Self::new();
+    for i in 0..min_length {
+      let current = unsafe { self.get_unchecked(i) };
+      match right.iter().find(|&item| item == current).is_some() {
+        true => (),
+        false => unsafe { res.push_unchecked((*current).clone()) },
+      }
+    }
+    res
+  }
+
+  /// Returns a new StaticVec representing the symmetric difference of `self` and `other` (that is,
+  /// all items present in at least one of `self` or `other`, but *not* present in both.)
+  ///
+  /// The `N2` parameter does not need to be provided explicitly, and can be inferred from `other`
+  /// itself.
+  ///
+  /// Locally requires that `T` implements [`Clone`](core::clone::Clone) to avoid soundness issues
+  /// while accommodating for more types than [`Copy`](core::marker::Copy) would appropriately for
+  /// this function, and [`PartialEq`](core::cmp::PartialEq) to make the item comparisons possible.
+  ///
+  /// Example usage:
+  /// ```
+  /// assert_eq!(
+  ///   staticvec![1, 2, 3].symmetric_difference(&staticvec![3, 4, 5]),
+  ///   [1, 2, 4, 5]
+  /// );
+  /// ```
+  #[inline]
+  pub fn symmetric_difference<const N2: usize>(
+    &self,
+    other: &StaticVec<T, N2>,
+  ) -> StaticVec<T, { N + N2 }>
+  where
+    T: Clone + PartialEq,
+  {
+    let min_length = self.length.min(other.length);
+    let left = self.as_slice();
+    let right = other.as_slice();
+    let mut res = StaticVec::new();
+    for i in 0..min_length {
+      let (current_left, current_right) =
+        unsafe { (self.get_unchecked(i), other.get_unchecked(i)) };
+      match right.iter().find(|&item| item == current_left).is_some() {
+        true => (),
+        false => unsafe { res.push_unchecked((*current_left).clone()) },
+      }
+      match left.iter().find(|&item| item == current_right).is_some() {
+        true => (),
+        false => unsafe { res.push_unchecked((*current_right).clone()) },
+      }
+    }
+    res
+  }
+
   #[doc(hidden)]
   #[inline(always)]
   pub(crate) fn new_data() -> [MaybeUninit<T>; N] {
