@@ -298,6 +298,15 @@ fn dedup_by_key() {
   assert_eq!(vec, [10, 20, 30, 20]);
 }
 
+#[test]
+fn difference() {
+  assert_eq!(
+    staticvec![4, 5, 6, 7].difference(&staticvec![1, 2, 3, 7]),
+    [4, 5, 6]
+  );
+  assert_eq!(staticvec![1, 2, 3].difference(&staticvec![3, 4, 5]), [1, 2]);
+}
+
 #[test]
 fn drain() {
   let mut v = staticvec![1, 2, 3];
@@ -828,6 +837,18 @@ fn split_off() {
   assert_eq!(vec2, [2, 3]);
 }
 
+#[test]
+fn symmetric_difference() {
+  assert_eq!(
+    staticvec![1, 2, 3].symmetric_difference(&staticvec![3, 4, 5]),
+    [1, 2, 4, 5]
+  );
+  assert_eq!(
+    staticvec![501, 502, 503, 504].symmetric_difference(&staticvec![502, 503, 504, 505]),
+    [501, 505]
+  );
+}
+
 #[test]
 fn swap_pop() {
   let mut v = staticvec!["foo", "bar", "baz", "qux"];
@@ -890,9 +911,10 @@ fn try_insert() {
 #[test]
 fn try_push() {
   let mut vec = staticvec![1, 2, 3, 4, 5];
-  assert_eq!(vec.try_push(2), Err("Insufficient remaining capacity!"));
+  let err = vec.try_push(2).unwrap_err();
+  assert_eq!(err.into_value(), 2);
   let mut vec2 = StaticVec::<i32, 4>::new_from_slice(&[1, 2, 3]);
-  vec2.push(3);
+  assert_eq!(vec2.try_push(3), Ok(()));
   assert_eq!(vec2, [1, 2, 3, 3]);
 }
 
